local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CleanUI_InitDeBuffs()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "Buffs");
    end

    local frame = CreateFrame("Frame");
    --frame:HookScript("OnUpdate", CleanUI_DeBuffsOnUpdate);
    frame:RegisterEvent("UNIT_AURA");
    
    frame:HookScript("OnEvent", CleanUI_DeBuffsOnEvent);

    CleanUI_CleanDeBuffs();
    CleanUI_StartDelay(2, CleanUI_CleanDeBuffs);

    CleanUI_DeBuffsHookEvents();
    
    CleanUI_StartDelay(3, CleanUI_UIParent_UpdateTopFramePositions);
end

function CleanUI_DeBuffsOnEvent(self, event, ...)
    if (event == "UNIT_AURA") then
        CleanUI_CleanDeBuffs();
    end
end

function CleanUI_DeBuffsHookEvents()
    hooksecurefunc("UIParent_UpdateTopFramePositions", CleanUI_UIParent_UpdateTopFramePositions);
end

function CleanUI_UIParent_UpdateTopFramePositions()
    BuffFrame:ClearAllPoints();
    BuffFrame:SetPoint("TOPRIGHT", MinimapCluster, "TOPLEFT", -45, -10);
end

local function calculateAction(button)
    if (button:GetName()) then
        local _, globalID = GetActionInfo(button.action);
        local spellName;
        
        if (globalID == 216698) then
            -- Seele ernten
            spellName = GetSpellInfo(216708);
        elseif (globalID == 137378) then
            -- Hurrikan in der Flasche
            spellName = GetSpellInfo(215294);
        else
            spellName = GetSpellInfo(globalID);
        end
        
        return spellName;
    end
    
    return nil;
end

function CleanUI_CleanDeBuffs()
    if (MASQUE and buttongroup) then
        CleanUI_SkinBuffs();
    end
end

function CleanUI_SkinBuffs()
    local button;
    local debuffType, color;

    for i=1, BUFF_MAX_DISPLAY do
        button = _G["BuffButton"..i];
        if (button) then
            if (MASQUE) then
                buttongroup:RemoveButton(button);
                buttongroup:AddButton(button);
            end
            
            -- resize buffs (default = 30)
            button:SetSize(38, 30);
            _G["BuffButton"..i.."Icon"]:SetTexCoord(0.0, 1.0, 0.1, 0.9);

            local duration = _G["BuffButton"..i.."Duration"];
            duration:ClearAllPoints();
            duration:SetPoint("TOP", button, "BOTTOM", 0, -3);
        end
    end

    for i=1, DEBUFF_MAX_DISPLAY do
        button = _G["DebuffButton"..i];
        if (button) then
            if (MASQUE) then
                buttongroup:RemoveButton(button);
                buttongroup:AddButton(button);
            end
            
            -- resize debuffs (default = 33)
            button:SetSize(38, 36);
            _G["DebuffButton"..i.."Icon"]:SetTexCoord(0.0, 1.0, 0.03, 0.97);

            local duration = _G["DebuffButton"..i.."Duration"];
            duration:ClearAllPoints();
            duration:SetPoint("TOP", button, "BOTTOM", 0, -3);
        end
    end
end

function CleanUI_ActionButtonDeBuffs_OnUpdate(self, elapsed)
    local spellName = calculateAction(self);
    CleanUI_DeBuffsUpdate(self, spellName);
end

---------------
-- Detect Unit Buffs
---------------

function CleanUI_UnitBuff(unit, spellName)
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, count, debuffType, duration, expirationTime, unitCaster = UnitBuff(unit, ii);
        
        if (name == nil) then
            return nil;
        end

        if (name and name == spellName) then
            return name, icon, count, debuffType, duration, expirationTime, unitCaster;
        end
    end
    
    return nil;
end

function CleanUI_CountUnitBuff(unit, spellName)
    local count = 0;

    local name, icon, stacks, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, stacks, debuffType, duration, expirationTime, unitCaster = UnitBuff(unit, ii);
        
        if (name == nil) then
            return count;
        end

        if (name and name == spellName and unitCaster == "player") then
            count = count + 1;
        end
    end
        
    return count;
end

function CleanUI_UnitDebuff(unit, spellName)
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, count, debuffType, duration, expirationTime, unitCaster = UnitDebuff(unit, ii);
        
        if (name == nil) then
            return nil;
        end

        if (name and name == spellName) then
            return name, icon, count, debuffType, duration, expirationTime, unitCaster;
        end
    end
    
    return nil;
end

function CleanUI_CountUnitDebuff(unit, spellName)
    local count = 0;

    local name, icon, stacks, debuffType, duration, expirationTime, unitCaster;
    for ii=1, 40 do 
        name, icon, stacks, debuffType, duration, expirationTime, unitCaster = UnitDebuff(unit, ii);
        
        if (name == nil) then
            return count;
        end

        if (name and name == spellName and unitCaster == "player") then
            count = count + 1;
        end
    end
        
    return count;
end

---------------
-- Update Unit Buffs
---------------

function CleanUI_DeBuffsUpdate(button, spellName)
    local statusbar, shown;
    local name, icon, count, debuffType, duration, expirationTime, unitCaster;
    local target, modifiedClick;

    if (spellName) then
        target = "target";
        modifiedClick = GetModifiedClick("FOCUSCAST");

        -- focus infos

        if (modifiedClick == "ALT" and IsAltKeyDown()) then
            target = "focus";
        elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
            target = "focus";
        elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
            target = "focus";
        end

        -- check buffs

        name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitBuff(target, spellName);

        if (name and unitCaster == "player") then
            shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);

            -- update de/buff counts
    
            local countUnitBuff = CleanUI_CountUnitBuff(target, spellName);
            CleanUI_UpdateDeBuffCountInfo(button, countUnitBuff);
        end

        if (not shown) then
            -- check debuffs
            name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitDebuff(target, spellName);    

            if (name and unitCaster == "player") then
                shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);

                -- update de/buff counts
        
                local countUnitDebuff = CleanUI_CountUnitDebuff(target, spellName);
                CleanUI_UpdateDeBuffCountInfo(button, countUnitDebuff);
            end
        end

        if (not shown) then
            -- self buffs
            name, icon, count, debuffType, duration, expirationTime, unitCaster = CleanUI_UnitBuff("player", spellName);

            if (name and unitCaster == "player") then
                shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);

                -- update de/buff counts
        
                local countUnitBuff = CleanUI_CountUnitBuff(target, spellName);
                CleanUI_UpdateDeBuffCountInfo(button, countUnitBuff);
            end
        end
    end

    if (not shown) then
        if (button) then
            local statusbar = CleanUI_GetStatusBar(button);
            statusbar:Hide();
            
            local stackText = CleanUI_GetDeBuffStack(button)
            stackText:SetText("");
        end
    end
end

function CleanUI_UpdateDeBuffCountInfo(button, count)
    local name = button:GetName().."DeBuffCount";
    local counttext = _G[name];

    local statusbar = CleanUI_GetStatusBar(button);
    
    if (not counttext) then
        counttext = statusbar:CreateFontString(name, "OVERLAY");
        counttext:SetFontObject("GameFontNormalHuge");
        counttext:ClearAllPoints();
        counttext:SetPoint("CENTER", 0, 0);
    end

    if (count > 1) then
        counttext:SetText(YELLOW_FONT_COLOR_CODE..count..FONT_COLOR_CODE_CLOSE);
    else
        counttext:SetText("");
    end
end

function CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count)
    if (not button) then
        return false;
    end

    local statusbar = CleanUI_GetStatusBar(button);
    statusbar:Show();

    local startTime = expirationTime - duration;

    if (expirationTime - startTime == 0) then
        statusbar:SetMinMaxValues(0, 100);
        statusbar:SetValue(100);
    else
        statusbar:SetMinMaxValues(0, expirationTime-startTime);
        statusbar:SetValue(expirationTime-GetTime());
    end
    
    local stackText = CleanUI_GetDeBuffStack(button)
    if (count > 0) then
      stackText:SetText(YELLOW_FONT_COLOR_CODE..count..FONT_COLOR_CODE_CLOSE);
    else
      stackText:SetText("");
    end

    return true;
end

function CleanUI_GetDeBuffStack(button)
    local name = button:GetName().."DeBuffStack";
    local fontString = _G[name];
    
    if (not fontString) then
        fontString = button:CreateFontString(name, "ARTWORK");
        fontString:SetFontObject("GameFontNormal");
        fontString:SetJustifyH("LEFT");
        fontString:SetJustifyV("TOP");
        fontString:ClearAllPoints();
        fontString:SetPoint("TOPLEFT", 2, -2);
    end

    return fontString;
end

function CleanUI_GetStatusBar(button)

    if (button.statusbar) then
        return button.statusbar;
    end

    button.statusbar = CreateFrame("StatusBar", nil, button, "CleanUIActionButtonDeBuffBarTemplate");

    button.statusbar:SetSize(button:GetWidth() - 2, 6);
    button.statusbar:ClearAllPoints();
    button.statusbar:SetPoint("BOTTOM", button, "BOTTOM", 0, 1);
    button.statusbar:SetStatusBarColor(1, 1, 0);
    button.statusbar:SetFrameStrata("HIGH");

    return button.statusbar;
end
